#include<menuet/os.h>
#include<stdio.h>
#include<string.h>
#include<stdint.h>

char eident[] = {0x7F, 'E', 'L', 'F', 0x0};


typedef short int Elf32_Half;
typedef int Elf32_Word;
typedef int Elf32_Addr; 
typedef int Elf32_Off; 

#define EI_NIDENT 16


struct elf_header { 
unsigned char e_ident[EI_NIDENT];
Elf32_Half e_type;
Elf32_Half e_machine;
Elf32_Word e_version;
Elf32_Addr e_entry;
Elf32_Off e_phoff;
Elf32_Off e_shoff;
Elf32_Word e_flags;
Elf32_Half e_ehsize;
Elf32_Half e_phentsize;
Elf32_Half e_phnum;
Elf32_Half e_shentsize;
Elf32_Half e_shnum;
Elf32_Half e_shstrndx;
} Elf32_Ehdr; 


struct elf_ph {
Elf32_Word p_type;
Elf32_Off p_offset;
Elf32_Addr p_vaddr;
Elf32_Addr p_paddr;
Elf32_Word p_filesz;
Elf32_Word p_memsz;
Elf32_Word p_flags;
Elf32_Word p_align;
};

 int (* __attribute__((cdecl)) gogogo)(void);

void static * useralloc(unsigned addr, unsigned s)
{
asm volatile ("int $0x80"::"a"(0), "b"(addr), "c"(s) );
}




int main(int argc, char *argv[]) {
    __menuet__debug_out("KolibriOS ELF loader\n");


  if (argc < 2) {
    __menuet__debug_out("Usage: elf progname.\n");
  }
  
  FILE *infile;
  infile = fopen(argv[1], "rb");
 
  if (infile == NULL) {
    __menuet__debug_out("File does not exist.\n");
    return -1;
  }
 
  fseek(infile, 0, SEEK_END);
  size_t file_size = ftell(infile);
  rewind(infile);
 
  char *buffer = (char*)malloc(file_size * sizeof(char));
  if (buffer == NULL) {
    fclose(infile);
    __menuet__debug_out("Malloc error.\n");
    return -1;
  }
  size_t bytes_read = fread(buffer, sizeof(char), file_size, infile);
  if (bytes_read != file_size) {
       __menuet__debug_out("Can't read enough bytes.\n");
    free(buffer);
    fclose(infile);
    return -1;
  }
  fclose(infile);
  
  if (strncmp(eident,buffer,4)!=0) {
	    __menuet__debug_out("Not valid ELF file.\n");
	    return -1;
  }
  
  memcpy(&Elf32_Ehdr, buffer, sizeof(Elf32_Ehdr));
  char deb[64];
  
  if (Elf32_Ehdr.e_machine!=3) {
	   __menuet__debug_out("Not x86 ELF.\n");
	    return -1;
  }
  
    if (Elf32_Ehdr.e_phnum==0) {
	   __menuet__debug_out("Not a program.\n");
	    return -1;
  }
  
  sprintf(deb, "PH Offset: %x Bin Address: %x\n",  Elf32_Ehdr.e_phoff, Elf32_Ehdr.e_entry);
  __menuet__debug_out(deb);

struct	elf_ph ProgramHeader[2];
	
	memcpy(&ProgramHeader, buffer+Elf32_Ehdr.e_phoff, sizeof(ProgramHeader)*2);
	 __menuet__debug_out("Header 1\n");
	
		  sprintf(deb, "Vaddr: %x Mem: %x Offset: %x Size:%x\n",  ProgramHeader[0].p_vaddr, ProgramHeader[0].p_memsz, ProgramHeader[0].p_offset, ProgramHeader[0].p_filesz);
  __menuet__debug_out(deb);
  
   __menuet__debug_out("Header 2\n");
	
	  sprintf(deb, "Vaddr: %x Mem: %x Offset: %x Size:%x\n",  ProgramHeader[1].p_vaddr, ProgramHeader[1].p_memsz, ProgramHeader[1].p_offset, ProgramHeader[1].p_filesz);
  __menuet__debug_out(deb);
  
  /*p_vaddr - адрес, куда грузить код
   *p_offset - откуда грузить
   *p_filesz - сколько байт грузить
   *p_memsz - сколько байт в памяти выделять
   * */
int pv0, pm0, pv1, pm1;
pv0= ProgramHeader[0].p_vaddr&0xFFFFF000;

pm0= ProgramHeader[0].p_memsz+0x1000;



//hack

pv1= ProgramHeader[1].p_vaddr&0xFFFFF000-ProgramHeader[0].p_vaddr&0xFFFFF000;
pm1= pv1+ProgramHeader[1].p_memsz+0x1000;


char *ps1 = (char*)useralloc( pv0, pm1 * sizeof(char));

  sprintf(deb, "Addr:%x Size:%x Section1:%x \n",  pv0, pm0, ps1);
  __menuet__debug_out(deb);



memcpy (ProgramHeader[0].p_vaddr, buffer+ProgramHeader[0].p_offset, ProgramHeader[0].p_filesz);



/*
pv1= ProgramHeader[1].p_vaddr&0xFFFFF000;
pm1= ProgramHeader[1].p_memsz+0x1000;

	 sprintf(deb, "AddrN: %x AddrAll: %x \n",  pv1, pv0+((pm0+0x1000)&0xFFFFF000));
  __menuet__debug_out(deb);


if (pv1<=pv0+((pm0+0x1000)&0xFFFFF000))


{

	pv1=pv0+((pm0+0x1000)&0xFFFFF000)+0x1000;
	__menuet__debug_out("SPLIT MEMORY AREAS!\n");
}
    sprintf(deb, "Addr:%x Size:%x Section2\n",  pv1, pm1);
  __menuet__debug_out(deb);
  
char *ps2 = (char*)useralloc( pv1, pm1 * sizeof(char));

*/
memcpy (ProgramHeader[1].p_vaddr, buffer+ProgramHeader[1].p_offset, ProgramHeader[1].p_filesz);

int xargc=argc-1;


//int xargv[2]={argv[1],argv[2]};
//int *z=&xargv[0];


__menuet__debug_out("Ready...Set...Go!\n\n\n");

gogogo = ( __attribute__((cdecl)) int (*)(void)) Elf32_Ehdr.e_entry;

int t=xargc;
//asm volatile ("pushl $0" );
//asm volatile ("pushl %%eax"::"a"(argv[2]) );
while (t>0) {

asm volatile ("pushl %%eax"::"a"(argv[t]) );
t=t-1;
}

asm volatile ("pushl %%eax"::"a"(xargc) );

asm volatile ("jmp %%eax"::"a"(gogogo) );


//gogogo();
  
}
  
